<?php

namespace Drupal\random_reference_formatter\Controller;

use Drupal\Core\Controller\ControllerBase;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Render\RendererInterface;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Response;

/**
 * Process the income data and returns a random entity.
 */
class RandomEntityController extends ControllerBase {

  /**
   * The entity manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The renderer service.
   *
   * @var \Drupal\Core\Render\RendererInterface
   */
  protected $renderer;

  /**
   * Constructs dependencies.
   *
   * PointsConvertController constructor.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity manager service.
   * @param \Drupal\Core\Render\RendererInterface $renderer
   *   The renderer service.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager, RendererInterface $renderer = NULL) {
    $this->entityTypeManager = $entity_type_manager;
    $this->renderer = $renderer;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static($container->get('entity_type.manager'), $container->get('renderer'));
  }

  /**
   * Get the random Entity.
   *
   * @return \Symfony\Component\HttpFoundation\JsonResponse|\Symfony\Component\HttpFoundation\Response
   *   The incoming request.
   *
   * @return \Symfony\Component\HttpFoundation\JsonResponse
   *   JSON response containing the content.
   */
  public function content(Request $request) {

    $candidatesGeneric = explode(',', $request->get('ids'));
    $viewMode = $request->get('view_mode');

    if (empty($candidatesGeneric)) {
      return $this->returnError();
    }

    $candidates = [];
    foreach ($candidatesGeneric as $idGeneric) {
      $id = strtok($idGeneric, '/');
      $entityType = strtok('');

      if (empty($idGeneric) || empty($entityType)) {
        continue;
      }

      $candidates[] = [
        'id' => $id,
        'entityType' => $entityType,
      ];
    }

    if (empty($viewMode)) {
      $viewMode = 'default';
    }

    $quantity = $request->get('quantity') ?: 1;

    try {

      $output = $dataToLoad = [];
      $randomEntities = array_intersect_key($candidates, $this->generateRandomNumbers(0, count($candidates) - 1, $quantity));

      foreach ($randomEntities as $randomEntityData) {
        $dataToLoad[$randomEntityData['entityType']][] = $randomEntityData['id'];
      }

      foreach ($dataToLoad as $entityType => $ids) {

        $entityStorage = $this->entityTypeManager
          ->getStorage($entityType);
        $entities = $entityStorage->loadMultiple($ids);

        $entityViewBuilder = $this->entityTypeManager
          ->getViewBuilder($entityType);
        $output += $entityViewBuilder->viewMultiple($entities, $viewMode);

      }

      $randomOutput = !empty($output) ? $this->renderer->render($output) : '';

    }
    catch (\Exception $e) {
      watchdog_exception('random_reference_formatter', $e);
      return new Response('', 500);
    }

    return new JsonResponse(['randomEntities' => $randomOutput]);
  }

  /**
   * Generate random numbers.
   *
   * @param int $min
   *   Min of random number.
   * @param int $max
   *   Max of random number.
   * @param int $quantity
   *   The number of random and unique items to generate.
   *
   * @return array
   *   An Array of random numbers.
   */
  private function generateRandomNumbers($min, $max, $quantity = 1): array {
    $randomNumbers = [];

    $j = 1;
    while ($j <= $quantity) {
      $rand = rand($min, $max);
      // Skip existing.
      if (in_array($rand, $randomNumbers)) {
        continue;
      }
      $randomNumbers[$rand] = $rand;
      $j++;
    }

    return $randomNumbers;
  }

  /**
   * Return error.
   *
   * @return \Symfony\Component\HttpFoundation\JsonResponse
   *   JSON response containing error.
   */
  private function returnError() {
    return new JsonResponse(['error' => TRUE]);
  }

}
